<?php

namespace Drupal\library_manager\Controller;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for library collection routes.
 */
class LibraryCollectionController extends ControllerBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The theme handler service.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $discovery;

  /**
   * Constructs the controller object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler service.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $discovery
   *   The discovery service.
   */
  public function __construct(ModuleHandlerInterface $module_handler, ThemeHandlerInterface $theme_handler, LibraryDiscoveryInterface $discovery) {
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
    $this->discovery = $discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('theme_handler'),
      $container->get('library.discovery')
    );
  }

  /**
   * Builds the response.
   */
  public function buildCollection() {

    $build['filters'] = [
      '#type' => 'search',
      '#title' => t('Filter libraries'),
      '#title_display' => 'invisible',
      '#size' => 30,
      '#placeholder' => t('Library name or extension'),
      '#attributes' => [
        'data-drupal-selector' => ['library-filter'],
        'autocomplete' => 'off',
      ],
    ];

    $header = [
      $this->t('Name'),
      $this->t('Extension'),
      $this->t('Version'),
      $this->t('License'),
      $this->t('Operations'),
    ];

    $rows = [];
    foreach ($this->getEnabledExtensions() as $extension) {
      $libraries = $this->discovery->getLibrariesByExtension($extension);
      foreach ($libraries as $library_name => $library) {

        $row = [];

        $row['name'] = $library_name;

        $row['extension'] = $extension;

        $row['version'] = isset($library['version']) ? $library['version'] : NULL;
        $row['license'] = isset($library['license']['name']) ? $library['license']['name'] : NULL;

        $links = [];
        $links['view'] = [
          'title' => $this->t('View'),
          'url' => Url::fromRoute('library_manager.library_canonnical', ['extension_id' => $extension, 'library_id' => urlencode($library_name)]),
        ];

        $row['links'] = [
          'data' => [
            '#type' => 'operations',
            '#links' => $links,
          ],
        ];

        $rows[] = $row;
      }
    }

    $build['table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
      '#attributes' => ['data-drupal-selector' => 'library-list'],
    ];

    $build['#attached']['library'][] = 'library_manager/library_manager';

    return $build;
  }

  /**
   * Builds autocomplete response for the library ID.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object that contains the typed keys.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The matched library ID as a JSON response.
   */
  public function buildAutocomplete(Request $request) {

    $matches = [];

    // Get the typed string from the URL, if it exists.
    if ($input = $request->query->get('q')) {
      foreach ($this->getEnabledExtensions() as $extension) {
        $libraries = $this->discovery->getLibrariesByExtension($extension);
        foreach ($libraries as $library_name => $library) {

          $library_id = $extension . '/' . $library_name;
          if (strpos($library_id, $input) !== FALSE) {
            $matches[] = [
              'label' => $library_id,
              'value' => $library_id,
            ];
          }

        }
      }
    }

    return new JsonResponse($matches);
  }

  /**
   * Finds enabled extensions.
   *
   * @return array
   *   List of enabled extension including core.
   */
  protected function getEnabledExtensions() {

    $enabled_extensions = ['core'];

    $listing = new ExtensionDiscovery(DRUPAL_ROOT);
    $listing->setProfileDirectories([]);

    $extensions = $listing->scan('module', TRUE);
    $extensions += $listing->scan('profile', TRUE);
    $extensions += $listing->scan('theme', TRUE);

    foreach ($extensions as $extension) {
      $extension_name = $extension->getName();

      // Do not check libraries for disabled extensions.
      if ($extension->getType() == 'theme') {
        if (!$this->themeHandler->themeExists($extension_name)) {
          continue;
        }
      }
      elseif (!$this->moduleHandler->moduleExists($extension_name)) {
        continue;
      }

      $enabled_extensions[] = $extension_name;
    }

    return $enabled_extensions;
  }

}
