<?php

namespace Drupal\library_manager\Controller;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for Library manager routes.
 */
class LibraryDetailsController extends ControllerBase {

  /**
   * The library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $discovery;

  /**
   * Constructs the controller object.
   *
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $discovery
   *   The discovery service.
   */
  public function __construct(LibraryDiscoveryInterface $discovery) {
    $this->discovery = $discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('library.discovery')
    );
  }

  /**
   * Builds the response.
   */
  public function build($extension_id, $library_id) {

    $library = $this->discovery->getLibraryByName($extension_id, urldecode($library_id));
    if (!$library) {
      throw new NotFoundHttpException();
    }

    $build['#title'] = $library_id;

    $build['id'] = [
      '#type' => 'item',
      '#title' => t('ID'),
      '#markup' => $library_id,
    ];

    if (!empty($library['version'])) {
      $build['version'] = [
        '#type' => 'item',
        '#title' => t('Version'),
        '#markup' => $library['version'],
      ];
    }

    if (!empty($library['license']['name'])) {
      $build['license'] = [
        '#type' => 'item',
        '#title' => t('License'),
        '#markup' => $library['license']['name'],
      ];
    }

    if (isset($library['remote'])) {
      $remote_url = Url::fromUri($library['remote']);
      $build['remote'] = [
        '#type' => 'item',
        '#title' => t('Remote URL'),
        '#markup' => Link::fromTextAndUrl($library['remote'], $remote_url)->toString(),
      ];
    }

    if ($library['js']) {
      $links = [];
      foreach ($library['js'] as $file) {
        $file_name = $file['data'];
        $file_uri = $file['type'] == 'file' ? 'base://' . $file_name : $file_name;

        $links[$file_name] = [
          'title' => $file_name,
          'url' => Url::fromUri($file_uri),
        ];
      }
      $build['js'] = [
        '#type' => 'fieldset',
        '#title' => t('JS'),
        'links' => [
          '#theme' => 'links',
          '#links' => $links,
        ],
      ];
    }

    if ($library['css']) {
      $links = [];
      foreach ($library['css'] as $file) {
        $file_name = $file['data'];
        $file_uri = $file['type'] == 'file' ? 'base://' . $file_name : $file_name;

        $links[$file_name] = [
          'title' => $file_name,
          'url' => Url::fromUri($file_uri),
        ];
      }
      $build['css'] = [
        '#type' => 'fieldset',
        '#title' => t('CSS'),
        'links' => [
          '#theme' => 'links',
          '#links' => $links,
        ],
      ];
    }

    if ($library['dependencies']) {
      $links = [];
      foreach ($library['dependencies'] as $dependency) {
        list($extension_id, $library_id) = explode('/', $dependency);
        $links[$library_id] = [
          'title' => $extension_id . '/' . $library_id,
          'url' => Url::fromRoute(
            'library_manager.library_canonnical',
            ['extension_id' => $extension_id, 'library_id' => $library_id]
          ),
        ];
      }
      $build['dependencies'] = [
        '#type' => 'fieldset',
        '#title' => t('Dependencies'),
        'links' => [
          '#theme' => 'links',
          '#links' => $links,
        ],
      ];

    }

    return $build;
  }

}
